Package: dendRoAnalyst
Type: Package
Date: 2026-05-18
Title: A Tool for Processing and Analyzing Dendrometer Data
Version: 0.1.6
Authors@R: c(
    person("Sugam", "Aryal", email = "sugam.aryal@fau.de", role = c("aut", "cre", "dtc")),
    person("Martin", "Häusser", email = "martin.haeusser@fau.de", role = "aut"),
    person("Jussi", "Grießinger", email = "jussi.griessinger@fau.de", role = "aut"),
    person("Ze-Xin", "Fan", email = "fanzexin@xtbg.org.cn", role = "aut"),
    person("Achim", "Bräuning", email = "achim.braeuning@fau.de", role = c("aut", "dgs")))
Maintainer: Sugam Aryal <sugam.aryal@fau.de>
Description: There are various functions for managing and cleaning data before the application of different approaches. This includes identifying and erasing sudden jumps in dendrometer data not related to environmental change, identifying the time gaps of recordings, and changing the temporal resolution of data to different frequencies. Furthermore, the package calculates daily statistics of dendrometer data, including the daily amplitude of tree growth. Various approaches can be applied to separate radial growth from daily cyclic shrinkage and expansion due to uptake and loss of stem water. In addition, it identifies periods of consecutive days with user-defined climatic conditions in daily meteorological data, then check what trees are doing during that period. 
License: GPL-3
Encoding: UTF-8
LazyData: true
Depends: R (>= 4.1.0), base
Imports: stats, tidyverse, dplyr, ggplot2, lubridate, readxl, tibble,
        tidyr, zoo, forecast, mgcv, minpack.lm, pspline, moments,
        signal, readr, boot, rlang, changepoint, WaveletComp
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-20 13:29:58 UTC; xe40husu
Author: Sugam Aryal [aut, cre, dtc],
  Martin Häusser [aut],
  Jussi Grießinger [aut],
  Ze-Xin Fan [aut],
  Achim Bräuning [aut, dgs]
Repository: CRAN
Date/Publication: 2026-05-20 14:40:02 UTC
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
Built: R 4.6.0; ; 2026-05-22 00:32:02 UTC; unix
