# Generated by roxygen2: do not edit by hand

S3method(plot,SC_output)
S3method(plot,SC_output_clim)
S3method(plot,ZG_output)
S3method(plot,ZG_output_clim)
S3method(plot,clim_twd_stats)
S3method(plot,clim_twd_test)
S3method(plot,daily_output)
S3method(plot,daily_output_clim)
S3method(plot,dm_detrended)
S3method(plot,dm_epoch)
S3method(plot,dm_growth_evaluation)
S3method(plot,dm_growth_fit)
S3method(plot,dm_na_interpolation)
S3method(plot,dm_standardized)
S3method(plot,dm_wavelet)
S3method(plot,dm_wavelet_coherence)
S3method(plot,dm_wavelet_reconstruct)
S3method(plot,mean_dm_detrended)
S3method(plot,mov.cor)
S3method(plot,mov.cor.boot)
S3method(plot,mov_cor_dm)
S3method(plot,network_interpolation)
S3method(plot,summary_mov_cor_dm)
S3method(print,dm_growth_fit)
S3method(print,mov_cor_dm)
S3method(print,summary.clim_twd_stats)
S3method(print,summary.clim_twd_test)
S3method(print,summary.dm_epoch)
S3method(print,summary.dm_growth_fit)
S3method(print,summary.dm_wavelet)
S3method(print,summary.dm_wavelet_reconstruct)
S3method(print,summary_mov_cor_dm)
S3method(summary,clim_twd_stats)
S3method(summary,clim_twd_test)
S3method(summary,dm_epoch)
S3method(summary,dm_growth_fit)
S3method(summary,dm_wavelet)
S3method(summary,dm_wavelet_reconstruct)
S3method(summary,mov_cor_dm)
export(clim.twd)
export(clim.twd.stats)
export(clim.twd.test)
export(daily.data)
export(dendro.resample)
export(dendro.truncate)
export(dm.detrend.fit)
export(dm.fit.gompertz)
export(dm.growth.evaluate)
export(dm.growth.fit)
export(dm.growth.fit.double)
export(dm.na.interpolation)
export(dm_add_climate)
export(dm_daily_clim)
export(dm_epoch_extract)
export(dm_epoch_test)
export(dm_event_climate)
export(dm_event_climate_summary)
export(dm_event_climate_test)
export(dm_event_times)
export(dm_join_daily_clim)
export(dm_join_phase_clim)
export(dm_join_subdaily_clim)
export(dm_plot_climate)
export(dm_plot_climate_compare)
export(dm_standardize)
export(dm_subdaily_clim)
export(dm_wavelet)
export(dm_wavelet_coherence)
export(dm_wavelet_reconstruct)
export(i.jump.locator)
export(jump.locator)
export(mean_detrended.dm)
export(mov.cor.dm)
export(network.interpolation)
export(phase.sc)
export(phase.zg)
export(plot_dm_assessment)
export(plot_dm_gaps)
export(plot_dm_interpolation)
export(plot_event_climate_box)
export(plot_event_climate_relation)
export(plot_mov.cor)
export(read.climate)
export(read.dendrometer)
export(reso_dm)
export(smooth_dm)
export(twd.maxima)
importFrom(boot,boot)
importFrom(changepoint,cpt.mean)
importFrom(changepoint,cpts)
importFrom(dplyr,"%>%")
importFrom(dplyr,`%>%`)
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,cur_column)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,first)
importFrom(dplyr,group_by)
importFrom(dplyr,if_else)
importFrom(dplyr,lag)
importFrom(dplyr,last)
importFrom(dplyr,lead)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,n_distinct)
importFrom(dplyr,relocate)
importFrom(dplyr,rename)
importFrom(dplyr,select)
importFrom(dplyr,slice_max)
importFrom(dplyr,summarise)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(dplyr,where)
importFrom(forecast,na.interp)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,facet_grid)
importFrom(ggplot2,facet_wrap)
importFrom(ggplot2,geom_area)
importFrom(ggplot2,geom_boxplot)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_contour)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_jitter)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_polygon)
importFrom(ggplot2,geom_raster)
importFrom(ggplot2,geom_rect)
importFrom(ggplot2,geom_ribbon)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_smooth)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,geom_violin)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_dodge)
importFrom(ggplot2,position_jitter)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_fill_viridis_c)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_x_date)
importFrom(ggplot2,scale_x_discrete)
importFrom(ggplot2,scale_x_log10)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,scale_y_log10)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlab)
importFrom(ggplot2,ylab)
importFrom(lubridate,floor_date)
importFrom(lubridate,hour)
importFrom(lubridate,minute)
importFrom(lubridate,minutes)
importFrom(lubridate,month)
importFrom(lubridate,parse_date_time)
importFrom(lubridate,second)
importFrom(lubridate,seconds)
importFrom(lubridate,yday)
importFrom(lubridate,year)
importFrom(lubridate,ymd)
importFrom(lubridate,ymd_hms)
importFrom(minpack.lm,nlsLM)
importFrom(moments,kurtosis)
importFrom(pspline,smooth.Pspline)
importFrom(readxl,read_excel)
importFrom(rlang,.data)
importFrom(stats,aov)
importFrom(stats,approx)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,kruskal.test)
importFrom(stats,lm)
importFrom(stats,loess)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.exclude)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pairwise.t.test)
importFrom(stats,pairwise.wilcox.test)
importFrom(stats,predict)
importFrom(stats,qlogis)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,residuals)
importFrom(stats,runmed)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,shapiro.test)
importFrom(stats,smooth.spline)
importFrom(stats,t.test)
importFrom(stats,ts)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyverse,tidyverse_packages)
importFrom(tools,file_ext)
importFrom(utils,head)
importFrom(utils,modifyList)
importFrom(utils,read.csv)
importFrom(utils,read.table)
importFrom(utils,tail)
importFrom(zoo,na.approx)
importFrom(zoo,na.locf)
importFrom(zoo,na.spline)
importFrom(zoo,rollapply)
