.onAttach <- function(libname, pkgname) {
  packageStartupMessage("Welcome to dendRoAnalyst 0.1.6!\n",
          "To check the changes in this version, check NEWS file.\n",
          "Please see the publication: Aryal, S., Haeusser, M., Griessinger, J., Fan, Z., and Braeuning, A. (2020). 'dendRoAnalyst': A tool for processing and analysing dendrometer data. Dendrochronologia, 64, 125772.")
}
