#' @title Daily rainfall data of Kathmandu for 2017.
#' @description This file contains daily rainfall data of Kathmandu. The source of this data is 'Government of Nepal, Department of Hydrology and Meteorology'.
#' @format A data frame with 365 rows and 2 variables:
#' \describe{
#'   \item{\code{TIME}}{Date in YYYY-MM-DD format.}
#'   \item{\code{rainfall}}{double rainfall in millimeters}
#'}
#' @source \url{http://www.mfd.gov.np/city?id=31/}
"ktm_rain17"
