% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_infer_double_richards_starts}
\alias{dmgfd_infer_double_richards_starts}
\title{Infer double-Richards starting values}
\usage{
dmgfd_infer_double_richards_starts(x, y, a0)
}
\arguments{
\item{x}{Season-day values.}

\item{y}{Growth values.}

\item{a0}{Initial total asymptote.}
}
\value{
Named list of starting values.
}
\description{
Infer double-Richards starting values
}
\keyword{internal}
