% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_extract_from_anchors}
\alias{dmea_extract_from_anchors}
\title{Extract climate values around anchor times}
\usage{
dmea_extract_from_anchors(anchor_times, clim_tbl, lag_values, vars)
}
\arguments{
\item{anchor_times}{Anchor times.}

\item{clim_tbl}{Aggregated climate table from [dmea_prepare_climate()].}

\item{lag_values}{Lag values in seconds.}

\item{vars}{Climate variables to extract.}
}
\value{
A long-format tibble.
}
\description{
Extract climate values around anchor times
}
\keyword{internal}
