% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_cummax_na}
\alias{dmgfd_cummax_na}
\title{Cumulative maximum while preserving missing values}
\usage{
dmgfd_cummax_na(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
Numeric vector.
}
\description{
Cumulative maximum while preserving missing values
}
\keyword{internal}
