% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_build_season_table}
\alias{dmgf_build_season_table}
\title{Build vegetation-season table}
\usage{
dmgf_build_season_table(
  time_vec,
  site_mode = c("NH", "SH", "CS"),
  custom_veg_season = c(275, 274)
)
}
\arguments{
\item{time_vec}{Time vector.}

\item{site_mode}{One of \code{"NH"}, \code{"SH"}, or \code{"CS"}.}

\item{custom_veg_season}{Custom vegetation season DOY range.}
}
\value{
Tibble with season labels and season timing.
}
\description{
Build vegetation-season table
}
\keyword{internal}
