% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_empty_model_params}
\alias{dmgf_empty_model_params}
\title{Empty model-parameter template}
\usage{
dmgf_empty_model_params()
}
\value{
Named list of empty model parameters.
}
\description{
Empty model-parameter template
}
\keyword{internal}
