% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_guess_two_pulses}
\alias{dmgfd_guess_two_pulses}
\title{Guess two pulse locations from growth increments}
\usage{
dmgfd_guess_two_pulses(x, y)
}
\arguments{
\item{x}{Season-day values.}

\item{y}{Cumulative growth values.}
}
\value{
Numeric vector of two initial pulse locations.
}
\description{
Guess two pulse locations from growth increments
}
\keyword{internal}
