% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_filter_time_table}
\alias{dmea_filter_time_table}
\title{Filter a table by calendar year and day-of-year}
\usage{
dmea_filter_time_table(dat, time_col, Year = NULL, DOY = NULL, label = "rows")
}
\arguments{
\item{dat}{A data frame or tibble.}

\item{time_col}{Name of the timestamp column.}

\item{Year}{Optional numeric vector of calendar years.}

\item{DOY}{Optional numeric vector of day-of-year values.}

\item{label}{Text used in error messages.}
}
\value{
A filtered tibble.
}
\description{
Filter a table by calendar year and day-of-year
}
\keyword{internal}
