% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{print.summary.dm_growth_fit}
\alias{print.summary.dm_growth_fit}
\title{Print a summary.dm_growth_fit object}
\usage{
\method{print}{summary.dm_growth_fit}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"summary.dm_growth_fit"}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
The input object, invisibly.
}
\description{
Prints a formatted summary of a \code{"summary.dm_growth_fit"} object.
}
