% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_pulse_window_from_rate}
\alias{dmgfd_pulse_window_from_rate}
\title{Estimate pulse window from fitted growth rate}
\usage{
dmgfd_pulse_window_from_rate(rate, x_pred, threshold, domain_idx)
}
\arguments{
\item{rate}{Fitted growth-rate vector.}

\item{x_pred}{Prediction season-day values.}

\item{threshold}{Pulse-specific rate threshold.}

\item{domain_idx}{Indices belonging to one pulse domain.}
}
\value{
List with start and end season-day.
}
\description{
Estimate pulse window from fitted growth rate
}
\keyword{internal}
