% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_growth_rate}
\alias{dmgfd_growth_rate}
\title{Calculate non-negative fitted growth rate}
\usage{
dmgfd_growth_rate(pred)
}
\arguments{
\item{pred}{Fitted cumulative growth.}
}
\value{
Numeric growth-rate vector.
}
\description{
Calculate non-negative fitted growth rate
}
\keyword{internal}
