% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_mov.cor.R
\name{plot_mov.cor}
\alias{plot_mov.cor}
\title{Backward-compatible wrapper for plotting moving correlation}
\usage{
plot_mov.cor(mov.cor.output, ...)
}
\arguments{
\item{mov.cor.output}{Output of \code{mov.cor.dm()}.}

\item{...}{Passed to \code{plot.mov_cor_dm()}.}
}
\value{
A \code{ggplot2} object.
}
\description{
Wrapper around \code{plot()} for objects returned by \code{mov.cor.dm()}.
}
