% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_apply_fun}
\alias{dmea_apply_fun}
\title{Apply an aggregation function}
\usage{
dmea_apply_fun(x, fun)
}
\arguments{
\item{x}{Numeric vector.}

\item{fun}{Function or character string. Supported character values are
\code{"mean"}, \code{"sum"}, \code{"max"}, \code{"min"}, and
\code{"median"}.}
}
\value{
Numeric scalar.
}
\description{
Apply an aggregation function
}
\keyword{internal}
