% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_empty_model_params}
\alias{dmgfd_empty_model_params}
\title{Empty parameter template for double growth fitting}
\usage{
dmgfd_empty_model_params()
}
\value{
Named list of model and timing parameters.
}
\description{
Empty parameter template for double growth fitting
}
\keyword{internal}
