% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_local_maxima}
\alias{dmgfd_local_maxima}
\title{Identify local maxima}
\usage{
dmgfd_local_maxima(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
Integer indices of local maxima.
}
\description{
Identify local maxima
}
\keyword{internal}
