% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{summary.dm_epoch}
\alias{summary.dm_epoch}
\title{Summarize a dm_epoch object}
\usage{
\method{summary}{dm_epoch}(object, ...)
}
\arguments{
\item{object}{Object of class \code{"dm_epoch"}.}

\item{...}{Unused.}
}
\value{
An object of class \code{"summary.dm_epoch"}.
}
\description{
Summarize a dm_epoch object
}
