% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_infer_logistic_starts}
\alias{dmgf_infer_logistic_starts}
\title{Infer logistic starting values}
\usage{
dmgf_infer_logistic_starts(x, y, a0)
}
\arguments{
\item{x}{Season-day values.}

\item{y}{Growth values.}

\item{a0}{Initial asymptote.}
}
\value{
Named list with \code{a}, \code{k}, and \code{t0}.
}
\description{
Infer logistic starting values
}
\keyword{internal}
