% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_stat_fun}
\alias{dmea_stat_fun}
\title{Composite statistic helper}
\usage{
dmea_stat_fun(x, statistic = c("mean", "median"))
}
\arguments{
\item{x}{Numeric vector.}

\item{statistic}{Statistic. One of \code{"mean"} or \code{"median"}.}
}
\value{
Numeric scalar.
}
\description{
Composite statistic helper
}
\keyword{internal}
