% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats.clim.twd.R
\name{summary.clim_twd_test}
\alias{summary.clim_twd_test}
\title{Summarize clim.twd.test output}
\usage{
\method{summary}{clim_twd_test}(object, ...)
}
\arguments{
\item{object}{An object of class \code{"clim_twd_test"}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
An object of class \code{"summary.clim_twd_test"}.
}
\description{
Summarize clim.twd.test output
}
