% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_unique_prediction_curve}
\alias{dmgf_unique_prediction_curve}
\title{Reduce prediction curve to unique x values}
\usage{
dmgf_unique_prediction_curve(x_pred, pred)
}
\arguments{
\item{x_pred}{Prediction x values.}

\item{pred}{Predicted y values.}
}
\value{
List with unique \code{x} and mean \code{pred}.
}
\description{
Reduce prediction curve to unique x values
}
\keyword{internal}
