% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_fit.R
\name{dmgp_prepare_curve_groups}
\alias{dmgp_prepare_curve_groups}
\title{Prepare curve grouping and faceting information}
\usage{
dmgp_prepare_curve_groups(dat, facet_by = c("default", "tree", "year", "none"))
}
\arguments{
\item{dat}{Tidy plotting data returned by [dmgp_tidy_fit_data()].}

\item{facet_by}{Character string controlling faceting layout.}
}
\value{
A list containing modified plotting data and faceting settings.
}
\description{
Internal helper used by [plot.dm_growth_fit()] to define panel identifiers,
facet variables, colour groups, and legend titles.
}
\keyword{internal}
