% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_doy_to_date}
\alias{dmgfd_doy_to_date}
\title{Convert year and day-of-year to Date}
\usage{
dmgfd_doy_to_date(year, doy)
}
\arguments{
\item{year}{Calendar year.}

\item{doy}{Day of year.}
}
\value{
Date.
}
\description{
Convert year and day-of-year to Date
}
\keyword{internal}
