% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_fit.R
\name{dmgp_prepare_timing_plot_data}
\alias{dmgp_prepare_timing_plot_data}
\title{Prepare timing-summary plot data}
\usage{
dmgp_prepare_timing_plot_data(
  fit_statistics,
  facet_by = c("default", "tree", "year", "none"),
  x_mode = c("date", "doy", "season_day"),
  series = NULL,
  fit_id = NULL
)
}
\arguments{
\item{fit_statistics}{Fit-statistics table from a \code{"dm_growth_fit"}
object.}

\item{facet_by}{Faceting mode.}

\item{x_mode}{Timing coordinate system. One of \code{"date"}, \code{"doy"},
or \code{"season_day"}.}

\item{series}{Optional series filter.}

\item{fit_id}{Optional fit identifier filter.}
}
\value{
A list of data frames used by the timing plot.
}
\description{
Internal helper used by [plot.dm_growth_fit()] to prepare the line segments,
point markers, and labels used in \code{type = "timing"} plots.
}
\keyword{internal}
