% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{print.summary.dm_epoch}
\alias{print.summary.dm_epoch}
\title{Print summary of a dm_epoch object}
\usage{
\method{print}{summary.dm_epoch}(x, ...)
}
\arguments{
\item{x}{Object of class \code{"summary.dm_epoch"}.}

\item{...}{Unused.}
}
\value{
The input object, invisibly.
}
\description{
Print summary of a dm_epoch object
}
