% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_detect_two_pulse_pattern}
\alias{dmgfd_detect_two_pulse_pattern}
\title{Detect two-pulse pattern from fitted growth rate}
\usage{
dmgfd_detect_two_pulse_pattern(
  rate,
  x_pred,
  peak_separation_min = 10,
  valley_ratio_max = 0.4,
  min_relative_peak = 0.05
)
}
\arguments{
\item{rate}{Fitted growth-rate vector.}

\item{x_pred}{Prediction season-day values.}

\item{peak_separation_min}{Minimum separation between peaks.}

\item{valley_ratio_max}{Maximum allowed valley-to-peak ratio.}

\item{min_relative_peak}{Minimum relative peak height.}
}
\value{
List describing detected pulse pattern.
}
\description{
Detect two-pulse pattern from fitted growth rate
}
\keyword{internal}
