% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.dm_growth_fit.R
\name{dmgp_resolve_x_axis}
\alias{dmgp_resolve_x_axis}
\title{Resolve x-axis mode for dendrometer growth-fit plots}
\usage{
dmgp_resolve_x_axis(type, x_axis = c("default", "season_day", "doy", "date"))
}
\arguments{
\item{type}{Plot type.}

\item{x_axis}{Requested x-axis representation.}
}
\value{
A list with elements \code{var}, \code{label}, and
  \code{timing_mode}.
}
\description{
Internal helper used by [plot.dm_growth_fit()] to determine which column is
used on the x-axis and which timing coordinate system should be used.
}
\keyword{internal}
