% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats.clim.twd.R
\name{print.summary.clim_twd_stats}
\alias{print.summary.clim_twd_stats}
\title{Print summary of clim.twd.stats output}
\usage{
\method{print}{summary.clim_twd_stats}(x, ...)
}
\arguments{
\item{x}{An object of class \code{"summary.clim_twd_stats"}.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
The input object, invisibly.
}
\description{
Print summary of clim.twd.stats output
}
