% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_add_overall_timing}
\alias{dmgfd_add_overall_timing}
\title{Add overall cumulative-growth timing}
\usage{
dmgfd_add_overall_timing(
  params,
  pred_full,
  x_pred,
  growth_fraction,
  season_start_date = as.Date(NA)
)
}
\arguments{
\item{params}{Parameter list.}

\item{pred_full}{Fitted cumulative growth.}

\item{x_pred}{Prediction season-day values.}

\item{growth_fraction}{Growth fractions.}

\item{season_start_date}{Season start date.}
}
\value{
Updated parameter list.
}
\description{
Add overall cumulative-growth timing
}
\keyword{internal}
