% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_unit_string}
\alias{dmea_unit_string}
\title{Create a lubridate period string}
\usage{
dmea_unit_string(step, unit)
}
\arguments{
\item{step}{Numeric step size.}

\item{unit}{Unit. One of \code{"mins"}, \code{"hours"}, or \code{"days"}.}
}
\value{
A character period string for [lubridate::floor_date()].
}
\description{
Create a lubridate period string
}
\keyword{internal}
