% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_fit_model_logistic}
\alias{dmgf_fit_model_logistic}
\title{Fit logistic growth curve}
\usage{
dmgf_fit_model_logistic(
  x,
  y,
  x_pred,
  fixed_a_value = NA_real_,
  fix_a_to_observed_max = FALSE,
  start_value_richards_parameters = list(a = NA_real_, k = NA_real_, t0 = NA_real_, v =
    1)
)
}
\arguments{
\item{x}{Observed x values.}

\item{y}{Observed y values.}

\item{x_pred}{Prediction x values.}

\item{fixed_a_value}{Optional fixed asymptote value.}

\item{fix_a_to_observed_max}{Logical.}

\item{start_value_richards_parameters}{Starting values.}
}
\value{
List with \code{pred} and \code{params}.
}
\description{
Fit logistic growth curve
}
\keyword{internal}
