% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_sea.R
\name{dmea_prepare_climate}
\alias{dmea_prepare_climate}
\title{Prepare climate data for superposed epoch analysis}
\usage{
dmea_prepare_climate(
  clim,
  vars = "all",
  step = 1,
  unit = c("hours", "days", "mins"),
  agg_fun = "mean",
  var_funs = NULL
)
}
\arguments{
\item{clim}{Climate data frame. First column must contain timestamps.}

\item{vars}{Climate variables to use, or \code{"all"}.}

\item{step}{Step size in units.}

\item{unit}{One of \code{"hours"}, \code{"days"}, or \code{"mins"}.}

\item{agg_fun}{Aggregation function applied to all variables when
\code{var_funs} is not supplied.}

\item{var_funs}{Optional named vector or named list of aggregation functions
for individual variables.}
}
\value{
A tibble with aggregated climate data.
}
\description{
Prepare climate data for superposed epoch analysis
}
\keyword{internal}
