% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_infer_gompertz_starts}
\alias{dmgfd_infer_gompertz_starts}
\title{Infer single Gompertz starting values}
\usage{
dmgfd_infer_gompertz_starts(x, y, a0)
}
\arguments{
\item{x}{Season-day values.}

\item{y}{Growth values.}

\item{a0}{Initial asymptote.}
}
\value{
Named list with \code{a}, \code{b}, and \code{k}.
}
\description{
Infer single Gompertz starting values
}
\keyword{internal}
