% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_add_anchor_points}
\alias{dmgf_add_anchor_points}
\title{Add zero-growth anchor points}
\usage{
dmgf_add_anchor_points(x, y, season_id = NULL)
}
\arguments{
\item{x}{Season-day values.}

\item{y}{Growth values.}

\item{season_id}{Optional season identifier for pooled fits.}
}
\value{
List with \code{x} and \code{y}.
}
\description{
Add zero-growth anchor points
}
\keyword{internal}
