% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_fit_model_loess}
\alias{dmgf_fit_model_loess}
\title{Fit loess growth curve}
\usage{
dmgf_fit_model_loess(x, y, x_pred, loess_span = 0.2, extrapolated = FALSE)
}
\arguments{
\item{x}{Observed x values.}

\item{y}{Observed y values.}

\item{x_pred}{Prediction x values.}

\item{loess_span}{Loess span.}

\item{extrapolated}{Logical flag for extrapolation.}
}
\value{
List with \code{pred} and \code{params}.
}
\description{
Fit loess growth curve
}
\keyword{internal}
