% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats.clim.twd.R
\name{plot.clim_twd_test}
\alias{plot.clim_twd_test}
\title{Plot method for clim.twd.test output}
\usage{
\method{plot}{clim_twd_test}(
  x,
  y = NULL,
  show_points = TRUE,
  facet = TRUE,
  legend_position = "none",
  main = NULL,
  ...
)
}
\arguments{
\item{x}{An object of class \code{"clim_twd_test"}.}

\item{y}{Unused.}

\item{show_points}{Logical. If \code{TRUE}, add jittered points.}

\item{facet}{Logical. If \code{TRUE}, facet by stratum when multiple strata exist.}

\item{legend_position}{Legend position.}

\item{main}{Optional plot title.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
A \code{ggplot2} object.
}
\description{
Plots the data used in \code{clim.twd.test()} as grouped boxplots with jittered
observations, faceted by stratum when applicable.
}
