% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm_wavelet_reconstruct.R
\name{print.summary.dm_wavelet_reconstruct}
\alias{print.summary.dm_wavelet_reconstruct}
\title{Print method for summary.dm_wavelet_reconstruct}
\usage{
\method{print}{summary.dm_wavelet_reconstruct}(x, digits = 4, ...)
}
\arguments{
\item{x}{An object of class \code{"summary.dm_wavelet_reconstruct"}.}

\item{digits}{Number of digits for rounded numeric printing.}

\item{...}{Further arguments passed to or from other methods.}
}
\value{
The input object, invisibly.
}
\description{
Print method for summary.dm_wavelet_reconstruct
}
