% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.na.interpolation.R
\name{plot_dm_interpolation}
\alias{plot_dm_interpolation}
\title{Plot original and interpolated dendrometer series}
\usage{
plot_dm_interpolation(
  x,
  original = NULL,
  series = NULL,
  start = NULL,
  end = NULL,
  free_y = TRUE,
  ncol = 1
)
}
\arguments{
\item{x}{Object returned by \code{dm.na.interpolation()}.}

\item{original}{Optional original input data frame used in
\code{dm.na.interpolation()}.}

\item{series}{Optional character vector of series names to plot.}

\item{start}{Optional start datetime for plotting subset.}

\item{end}{Optional end datetime for plotting subset.}

\item{free_y}{Logical. If \code{TRUE}, each facet gets its own y-scale.}

\item{ncol}{Integer. Number of facet columns.}
}
\value{
A \code{ggplot2} object.
}
\description{
Plot original and interpolated dendrometer series
}
\examples{
\donttest{
#res <- dm.na.interpolation(
#  nepa17[1:1000, ],
#  resolution = 60,
#  fill = TRUE,
#  method = "spline"
#)
#plot_dm_interpolation(res, original = nepa17[1:1000, ])
}
}
