% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_first_non_na}
\alias{dmgf_first_non_na}
\title{Return the first non-missing value}
\usage{
dmgf_first_non_na(x)
}
\arguments{
\item{x}{Numeric vector.}
}
\value{
First non-missing value, or \code{NA_real_}.
}
\description{
Return the first non-missing value
}
\keyword{internal}
