% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.R
\name{dmgf_is_scalar_finite}
\alias{dmgf_is_scalar_finite}
\title{Check whether a value is a finite numeric scalar}
\usage{
dmgf_is_scalar_finite(x)
}
\arguments{
\item{x}{Object to test.}
}
\value{
Logical scalar.
}
\description{
Check whether a value is a finite numeric scalar
}
\keyword{internal}
