% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dm.growth.fit.double.R
\name{dmgfd_fit_single_fallback}
\alias{dmgfd_fit_single_fallback}
\title{Fit single-curve fallback}
\usage{
dmgfd_fit_single_fallback(
  method,
  x,
  y,
  x_pred,
  fixed_a_value = NA_real_,
  fix_a_to_observed_max = FALSE,
  start_value_double_gompertz_parameters,
  start_value_double_richards_parameters
)
}
\arguments{
\item{method}{Original double method.}

\item{x}{Observed season-day values.}

\item{y}{Observed cumulative growth values.}

\item{x_pred}{Prediction season-day values.}

\item{fixed_a_value}{Optional fixed asymptote.}

\item{fix_a_to_observed_max}{Logical fixed-asymptote flag.}

\item{start_value_double_gompertz_parameters}{Double-Gompertz starts.}

\item{start_value_double_richards_parameters}{Double-Richards starts.}
}
\value{
List with \code{pred} and \code{params}.
}
\description{
Fit single-curve fallback
}
\keyword{internal}
